@@ -1,20 +1,21 @@
 # CHNOSZ/info.R
-# search database for species names or formulas
+# Search database for species names or formulas
 # and retrieve thermodynamic properties of species
-# 20061024 extracted from species.R jmd
-# 20120507 code rewrite and split into info.[character,approx,numeric];
-#   these functions expect arguments of length 1; 
+# 20061024 Extracted from species.R jmd
+# 20120507 Code rewrite and split into info.[character,approx,numeric];
+#   These functions expect arguments of length 1; 
 #   info() handles longer arguments
 
 ## If this file is interactively sourced, the following are also needed to provide unexported functions:
 #source("util.data.R")
 
 info <- function(species=NULL, state=NULL, check.it=TRUE) {
-  ## return information for one or more species in thermo()$OBIGT
+
+  ## Return information for one or more species in thermo()$OBIGT
   thermo <- get("thermo", CHNOSZ)
-  # that should give us the data, not the thermo() function 20190928
+  # That should give us the data, not the thermo() function 20190928
   if(is.function(thermo)) stop("CHNOSZ package data is not available; use reset() or library(CHNOSZ) to load it")
-  ## if no species are requested, summarize the available data  20101129
+  ## If no species are requested, summarize the available data  20101129
   if(is.null(species)) {
     message("info: 'species' is NULL; summarizing information about thermodynamic data...")
     message(paste("thermo()$OBIGT has", nrow(thermo$OBIGT[thermo$OBIGT$state=="aq", ]), "aqueous,",
@@ -25,85 +26,93 @@ info <- function(species=NULL, state=NULL, check.it=TRUE) {
       length(unique(thermo$protein$organism)), "organisms"))
     return()
   }
-  ## run info.numeric or info.character depending on the input type
+
+  ## Run info.numeric or info.character depending on the input type
   if(is.numeric(species)) {
+
     out <- lapply(species, info.numeric, check.it)
-    # if we have different states the column names could be different
+    # If we have different states the column names could be different
     if(length(unique(unlist(lapply(out, names)))) > ncol(thermo$OBIGT)) {
       # make them the same as thermo$OBIGT
       out <- lapply(out, function(row) {
         colnames(row) <- colnames(thermo$OBIGT); return(row)
       }) 
     }
-    # turn the list into a data frame
+    # Turn the list into a data frame
     out <- do.call(rbind, out)
-    # ensure that the rownames are numeric values (not names possibly inherited from retrieve()) 20190224
+    # Ensure that the rownames are numeric values (not names possibly inherited from retrieve()) 20190224
     if(!is.null(attr(species, "names"))) row.names(out) <- species
+
   } else {
-    # state and species should be same length
+
+    # State and species should be same length
     if(!is.null(state)) {
       lmax <- max(length(species), length(state))
       state <- rep(state, length.out=lmax)
       species <- rep(species, length.out=lmax)
     }
-    # loop over the species
+    # Loop over the species
     out <- sapply(seq_along(species), function(i) {
-      # first look for exact match
+      # First look for exact match
       ispecies <- info.character(species[i], state[i])
-      # if no exact match and it's not a protein, show approximate matches (side effect of info.approx)
+      # If no exact match and it's not a protein, show approximate matches (side effect of info.approx)
       if(identical(ispecies, NA) & !grepl("_", species[i])) ispecies.notused <- info.approx(species[i], state[i])
-      # do not accept multiple matches
+      # Do not accept multiple matches
       if(length(ispecies) > 1) ispecies <- NA
       return(ispecies)
     })
+
   }
-  ## all done!
+
+  ## All done!
   return(out)
 }
 
-### unexported functions ###
+### Unexported functions ###
 
-info.text <- function(ispecies) {
+info.text <- function(ispecies, withsource = TRUE) {
   # a textual description of species name, formula, source, e.g.
   # CO2 [CO2(aq)] (SSW01, SHS89, 11.Oct.07)
   this <- get("thermo", CHNOSZ)$OBIGT[ispecies, ]
+  out <- paste(this$name, " [", this$formula, "(", this$state, ")", "]", sep="")
+  if(!withsource) return(out)
   sourcetext <- this$ref1
   ref2 <- this$ref2
   if(!is.na(ref2)) sourcetext <- paste(sourcetext, ref2, sep=", ")
   date <- this$date
   if(!is.na(date)) sourcetext <- paste(sourcetext, date, sep=", ")
-  out <- paste(this$name, " [", this$formula, "(", this$state, ")", "] (", sourcetext, ")", sep="")
+  out <- paste(out, " (", sourcetext, ")", sep = "")
   return(out)
 }
 
 info.character <- function(species, state=NULL, check.protein=TRUE) {
-  # returns the rownumbers of thermo()$OBIGT having an exact match of 'species' to
+  # Return the rownumbers of thermo()$OBIGT having an exact match of 'species' to
   # thermo()$OBIGT$[species|abbrv|formula] or NA otherwise
-  # a match to thermo()$OBIGT$state is also required if 'state' is not NULL
-
+  # A match to thermo()$OBIGT$state is also required if 'state' is not NULL
   # (first occurence of a match to species is returned otherwise)
+
   thermo <- get("thermo", CHNOSZ)
-  # find matches for species name, abbreviation or formula
+  # Find matches for species name, abbreviation or formula
   matches.species <- thermo$OBIGT$name==species | thermo$OBIGT$abbrv==species | thermo$OBIGT$formula==species
-  # since thermo()$OBIGT$abbrv contains NAs, convert NA results to FALSE
+  # Since thermo()$OBIGT$abbrv contains NAs, convert NA results to FALSE
   matches.species[is.na(matches.species)] <- FALSE
-  # turn it in to no match if it's a protein in the wrong state
+  # Turn it in to no match if it's a protein in the wrong state
   ip <- pinfo(species)
   if(any(matches.species) & !is.na(ip) & !is.null(state)) {
     matches.state <- matches.species & grepl(state, thermo$OBIGT$state)
     if(!any(matches.state)) matches.species <- FALSE
   }
-  # no match, not available
+  # No match, not available
   if(!any(matches.species)) {
-    # unless it's a protein
+    # Unless it's a protein
     if(check.protein) {
-      # did we find a protein? add its properties to OBIGT
+      # Did we find a protein? add its properties to OBIGT
       if(!is.na(ip)) {
-        # here we use a default state from thermo()$opt$state
+        # Here we use a default state from thermo()$opt$state
         if(is.null(state)) state <- thermo$opt$state
-        # add up protein properties
+        # Add up protein properties
         eos <- protein.OBIGT(ip, state=state)
-        # the real assignment work 
+        # The real assignment work 
         nrows <- suppressMessages(mod.OBIGT(eos))
         thermo <- get("thermo", CHNOSZ)
         matches.species <- rep(FALSE, nrows)
@@ -111,20 +120,20 @@ info.character <- function(species, state=NULL, check.protein=TRUE) {
       } else return(NA)
     } else return(NA)
   }
-  # do we demand a particular state
+  # Do we demand a particular state
   if(!is.null(state)) {
-    # special treatment for H2O: aq retrieves the liq
+    # Special treatment for H2O: aq retrieves the liq
     if(species %in% c("H2O", "water") & state=="aq") state <- "liq"
-    # the matches for both species and state
+    # The matches for both species and state
     matches.state <- matches.species & state == thermo$OBIGT$state
     if(!any(matches.state)) {
-      # the requested state is not available for this species
+      # The requested state is not available for this species
       available.states <- thermo$OBIGT$state[matches.species]
       if(length(available.states)==1) a.s.verb <- "is" else a.s.verb <- "are"
       a.s.text <- paste("'", available.states, "'", sep="", collapse=" ")
       message("info.character: requested state '", state, "' for ", species, 
         " but only ", a.s.text, " ", a.s.verb, " available")
-      # warn about looking for aqueous methane (changed to CH4) 20200707
+      # Warn about looking for aqueous methane (changed to CH4) 20200707
       if(identical(species, "methane") & identical(state, "aq")) {
         warning("'methane' is not an aqueous species; use 'CH4' instead\nTo revert to the old behavior, run mod.OBIGT(info('CH4'), name = 'methane')")
       }
@@ -132,15 +141,15 @@ info.character <- function(species, state=NULL, check.protein=TRUE) {
     }
     matches.species <- matches.state
   }
-  # all of the species that match
+  # All of the species that match
   ispecies.out <- ispecies <- which(matches.species)
-  # processing for more than one match
+  # Processing for more than one match
   if(length(ispecies) > 1) {
-    # if a single name matches, use that one (useful for distinguishing pseudo-H4SiO4 and H4SiO4) 20171020
+    # If a single name matches, use that one (useful for distinguishing pseudo-H4SiO4 and H4SiO4) 20171020
     matches.name <- matches.species & thermo$OBIGT$name==species
     if(sum(matches.name)==1) ispecies.out <- which(matches.name)
     else ispecies.out <- ispecies[1]  # otherwise, return only the first species that matches
-    # let user know if there is more than one state for this species
+    # Let user know if there is more than one state for this species
     mystate <- thermo$OBIGT$state[ispecies.out]
     ispecies.other <- ispecies[!ispecies %in% ispecies.out]
     otherstates <- thermo$OBIGT$state[ispecies.other]
@@ -149,10 +158,10 @@ info.character <- function(species, state=NULL, check.protein=TRUE) {
       otherstates[otherstates==mystate] <- thermo$OBIGT$name[ispecies.other[otherstates==mystate]]
     }
     transtext <- othertext <- ""
-    # we count, but don't show the states for phase transitions (cr2, cr3, etc)
+    # We count, but don't show the states for phase transitions (cr2, cr3, etc)
     istrans <- otherstates %in% c("cr2", "cr3", "cr4", "cr5", "cr6", "cr7", "cr8", "cr9")
     if(mystate=="cr") {
-      # if we are "cr" we show the number of phase transitions
+      # If we are "cr" we show the number of phase transitions
       ntrans <- sum(istrans)
       if(ntrans == 1) transtext <- paste(" with", ntrans, "phase transition")
       else if(ntrans > 1) transtext <- paste(" with", ntrans, "phase transitions")
@@ -176,8 +185,8 @@ info.character <- function(species, state=NULL, check.protein=TRUE) {
 }
 
 info.numeric <- function(ispecies, check.it=TRUE) {
-  # from a numeric species index in 'ispecies' return the 
-  # thermodynamic properties and equations-of-state parameters
+  # From a numeric species index in 'ispecies' return the 
+  #   thermodynamic properties and equations-of-state parameters
   thermo <- get("thermo", CHNOSZ)
   # if we're called with NA, return an empty row
   if(is.na(ispecies)) {
@@ -186,7 +195,7 @@ info.numeric <- function(ispecies, check.it=TRUE) {
     return(this)
   }
   this <- thermo$OBIGT[ispecies,]
-  # species indices must be in range
+  # Species indices must be in range
   ispeciesmax <- nrow(thermo$OBIGT)
   if(ispecies > ispeciesmax | ispecies < 1) 
     stop(paste("species index", ispecies, "not found in thermo()$OBIGT\n"))
@@ -195,7 +204,11 @@ info.numeric <- function(ispecies, check.it=TRUE) {
   # Use new OBIGT2eos function here
   this <- OBIGT2eos(this, this$state)
 
-  if(all(is.na(this[, 9:21])) & this$name != "water") {
+  # Stop with an informative message if species don't have a model 20220929
+  namodel <- is.na(this$model)
+  if(namodel) stop(paste("Species has NA model:", info.text(ispecies, FALSE)), call. = FALSE)
+
+  if(tolower(this$model) == "berman") { # this is Berman
     # Get G, H, S, and V for minerals with Berman parameters 20220203
     Bermandat <- Berman()
     Bermandat <- Bermandat[Bermandat$name == this$name, ]
@@ -204,62 +217,60 @@ info.numeric <- function(ispecies, check.it=TRUE) {
   } else isBerman <- FALSE
 
   # Identify any missing GHS values
-  naGHS <- is.na(this[9:11])
+  naGHS <- is.na(this[10:12])
   # A missing one of G, H or S can cause problems for subcrt calculations at high T
   if(sum(naGHS)==1) {
     # calculate a single missing one of G, H, or S from the others
-    GHS <- as.numeric(GHS(as.character(this$formula), G=this[,9], H=this[,10], S=this[,11], E_units=this$E_units))
-    message("info.numeric: ", colnames(this)[9:11][naGHS], " of ",
+    GHS <- as.numeric(GHS(as.character(this$formula), G = this[, 10], H = this[, 11], S = this[, 12], E_units=this$E_units))
+    message("info.numeric: ", colnames(this)[10:12][naGHS], " of ",
       this$name, "(", this$state, ") is NA; set to ", round(GHS[naGHS],2), " ", this$E_units, " mol-1")
-    this[, which(naGHS)+8] <- GHS[naGHS]
+    this[, which(naGHS) + 9] <- GHS[naGHS]
   } 
 
   # Perform consistency checks for GHS and EOS parameters if check.it = TRUE
   # Don't do it for the AD species 20190219
-  if(check.it & !"xi" %in% colnames(this)) {
+  if(check.it & this$model != "AD") {
     # Check GHS if they are all present
     if(sum(naGHS)==0) calcG <- checkGHS(this)
     # Check tabulated heat capacities against EOS parameters
-    calcCp <- checkEOS(this, this$state, "Cp")
-    # fill in NA heat capacity
+    calcCp <- checkEOS(this, this$model, "Cp")
+    # Fill in NA heat capacity
     if(!is.na(calcCp) & is.na(this$Cp)) {
       message("info.numeric: Cp of ", this$name, "(", this$state, ") is NA; set by EOS parameters to ", round(calcCp, 2), " ", this$E_units, " K-1 mol-1")
       this$Cp <- as.numeric(calcCp)
     } else if(isBerman) {
       # Calculate Cp for Berman minerals 20220208
       calcCp <- Berman(this$name)$Cp
-      if(this$E_units == "J") calcCp <- convert(calcCp, "J")
       this$Cp <- calcCp
     }
-    # check tabulated volumes - only for aq (HKF equation)
-    if(identical(this$state, "aq")) {
-      calcV <- checkEOS(this, this$state, "V")
-      # fill in NA volume
+    # Check tabulated volumes - only for HKF model (aq species)
+    if(this$model %in% c("HKF", "DEW")) {
+      calcV <- checkEOS(this, this$model, "V")
+      # Fill in NA volume
       if(!is.na(calcV) & is.na(this$V)) {
         message("info.numeric: V of ", this$name, "(", this$state, ") is NA; set by EOS parameters to ", round(calcV, 2), " cm3 mol-1")
         this$V <- as.numeric(calcV)
       }
     }
-  } # done checking
-  # all done!
+  } # Done checking
+  # All done!
   return(this)
 }
 
 info.approx <- function(species, state=NULL) {
-  # returns species indices that have an approximate match of 'species'
-  # to thermo$OBIGT$[name|abbrv|formula], 
-  # possibly restricted to a given state
+  # Returns species indices that have an approximate match of 'species'
+  # to thermo$OBIGT$[name|abbrv|formula], possibly restricted to a given state
   thermo <- get("thermo", CHNOSZ)
   if(!is.null(state)) this <- thermo$OBIGT[thermo$OBIGT$state==state, ]
   else this <- thermo$OBIGT
-  # only look for fairly close matches
+  # Only look for fairly close matches
   max.distance <- 0.1
   approx.name <- agrep(species, this$name, max.distance)
   approx.abbrv <- agrep(species, this$abbrv, max.distance)
   approx.formula <- agrep(species, this$formula, max.distance)
   approx.species <- unique(c(approx.name, approx.abbrv, approx.formula))
   if(!is.na(approx.species[1])) {
-    # show the names of the species
+    # Show the names of the species
     if(length(approx.species)==1) {
       message("info.approx: '", species, "' is similar to ", info.text(approx.species))
     } else {
@@ -273,7 +284,7 @@ info.approx <- function(species, state=NULL) {
     }
     return(approx.species)
   }
-  # if we got here there were no approximate matches
+  # If we got here there were no approximate matches
   # 20190127 look for the species in optional data files 
   for(opt in c("SLOP98", "SUPCRT92", "AD")) {
     optdat <- read.csv(system.file(paste0("extdata/OBIGT/", opt, ".csv"), package="CHNOSZ"), as.is=TRUE)
