@@ -2,9 +2,9 @@
 # 20181031 first version jmd
 # 20181106 work on the output from affinity(); no "equilibrate()" needed!
 # 20190117 add find.IS and test for dissociation reaction
-# 20210319 use list of aqueous species as main argument (with back-compatibility for affinity output) and handle multiple minerals
+# 20210319 use vector of aqueous species as main argument (with back-compatibility for affinity output) and handle multiple minerals
 
-## if this file is interactively sourced, the following are also needed to provide unexported functions:
+## If this file is interactively sourced, the following are also needed to provide unexported functions:
 #source("equilibrate.R")
 #source("util.misc.R")
 #source("species.R")
@@ -28,7 +28,7 @@ solubility <- function(iaq, ..., in.terms.of = NULL, dissociate = FALSE, find.IS
     is.mosaic <- TRUE
     # For getting 'sout' from affinity(), drop arguments specific for mosaic()
     affargs <- ddd[-1]
-    affargs <- affargs[!names(affargs) %in% c("bases", "stable", "blend")]
+    affargs <- affargs[!names(affargs) %in% c("bases", "stable", "blend", "loga_aq")]
   }
 
   # Save current thermodynamic system settings
@@ -100,67 +100,67 @@ solubility <- function(iaq, ..., in.terms.of = NULL, dissociate = FALSE, find.IS
 # The "nuts and bolts" of solubility calculations
 # Moved from solubility() to solubility_calc() 20210318
 solubility_calc <- function(aout, dissociate = NULL, find.IS = FALSE, in.terms.of = NULL) {
-  ## concept: the logarithms of activities of species at equilibrium are equal to
+  ## Concept: the logarithms of activities of species at equilibrium are equal to
   ## Astar, the affinities calculated for unit activities of species
 
-  ## is aout the output from mosaic() instead of affinity()?
+  ## Is aout the output from mosaic() instead of affinity()?
   aout.save <- aout
   thisfun <- aout$fun
   if(thisfun=="mosaic") aout <- aout$A.species
 
-  # get starting ionic strength (probably zero, but could be anything set by user)
+  # Get starting ionic strength (probably zero, but could be anything set by user)
   IS <- aout$IS
   if(is.null(IS)) IS <- 0
 
-  ## to output loga.balance we need the balancing coefficients
+  ## To output loga.balance we need the balancing coefficients
   bout <- balance(aout)
   n.balance <- bout$n.balance
   balance <- bout$balance
-  # get logarithm of total activity of the conserved basis species
+  # Get logarithm of total activity of the conserved basis species
   logabfun <- function(loga.equil, n.balance) {
-    # exponentiate, multiply by n.balance, sum, logarithm
+    # Exponentiate, multiply by n.balance, sum, logarithm
     a.equil <- mapply("^", 10, loga.equil, SIMPLIFY = FALSE)
     a.balance <- mapply("*", a.equil, n.balance, SIMPLIFY = FALSE)
     a.balance <- Reduce("+", a.balance)
     log10(a.balance)
   }
   
-  # for find.IS=TRUE, iterate to converge on ionic strength
+  # For find.IS = TRUE, iterate to converge on ionic strength
   niter <- 1
   while(TRUE) {
-    ## the values in aout can be calculated for other than
+    ## The values in aout can be calculated for other than
     ## unit activities of species, so we have to take away the activites
     Astar <- function(i) aout$values[[i]] + aout$species$logact[i]
     loga.equil <- lapply(1:length(aout$values), Astar)
 
-    ## for a dissociation on a *per reaction* (not system) basis, apply the divisor here
+    ## For a dissociation on a *per reaction* (not system) basis, apply the divisor here
     ## (can be used to reproduce Fig. 4 of Manning et al., 2013)
     if(is.numeric(dissociate)) {
       loga.equil <- lapply(loga.equil, "/", dissociate)
     }
 
-    # get the total activity of the balancing basis species
+    # Get the total activity of the balancing basis species
     loga.balance <- logabfun(loga.equil, n.balance)
 
-    # recalculate things for a dissociation reaction (like CaCO3 = Ca+2 + CO3+2)
+    # Recalculate things for a dissociation reaction (like CaCO3 = Ca+2 + CO3+2)
     if(isTRUE(dissociate)) {
       ndissoc <- 2
-      # the number of dissociated products is the exponent in the activity product
+      # The number of dissociated products is the exponent in the activity product
       loga.dissoc <- loga.balance / ndissoc
-      # the contribution to affinity
+      # The contribution to affinity
       Adissoc <- lapply(n.balance, "*", loga.dissoc)
-      # adjust the affinity and get new equilibrium activities
+      # Adjust the affinity and get new equilibrium activities
       aout$values <- mapply("-", aout$values, Adissoc, SIMPLIFY = FALSE)
       loga.equil <- lapply(1:length(aout$values), Astar)
       loga.balance <- logabfun(loga.equil, n.balance)
-      # check that the new loga.balance == loga.dissoc
+      # Check that the new loga.balance == loga.dissoc
       # TODO: does this work for non-1:1 species?
       stopifnot(all.equal(loga.balance, loga.dissoc))
     }
 
-    # get the old IS
+    # Get the old IS
     IS.old <- rep(IS, length.out = length(aout$values[[1]]))
-    # calculate the ionic strength for unpaired ions: IS = sum(molality * Z^2) / 2
+    # Calculate the ionic strength for unpaired ions: IS = sum(molality * Z^2) / 2
     sum.mZ2 <- 0
     ion.names <- character()
     # is there an ion in the basis species?
@@ -172,8 +172,8 @@ solubility_calc <- function(aout, dissociate = NULL, find.IS = FALSE, in.terms.o
         ion.names <- c(ion.names, basis.ion)
       }
     }
-    # add ions present in the species of interest
-    # instead of using aout$species$name, use info() to get formulas 20190309
+    # Add ions present in the species of interest
+    # Instead of using aout$species$name, use info() to get formulas 20190309
     species.formulas <- suppressMessages(info(aout$species$ispecies, check.it = FALSE)$formula)
     for(i in 1:length(loga.equil)) {
       species.ion <- species.formulas[i]
@@ -184,15 +184,15 @@ solubility_calc <- function(aout, dissociate = NULL, find.IS = FALSE, in.terms.o
       }
     }
     IS <- sum.mZ2 / 2
-    # report ions used in the ionic strength calculation
-    if(find.IS & niter==1) message("solubility: ionic strength calculated for ", paste(ion.names, collapse=" "))
-    # report the current ionic strength
-    if(find.IS) message("solubility: (iteration ", niter, ") ionic strength range is ", paste(round(range(IS), 4), collapse=" "))
-    # stop iterating if we reached the tolerance (or find.IS=FALSE)
+    # Report ions used in the ionic strength calculation
+    if(find.IS & niter == 1) message("solubility: ionic strength calculated for ", paste(ion.names, collapse = " "))
+    # Report the current ionic strength
+    if(find.IS) message("solubility: (iteration ", niter, ") ionic strength range is ", paste(round(range(IS), 4), collapse = " "))
+    # Stop iterating if we reached the tolerance (or find.IS = FALSE)
     if(!find.IS | all(IS - IS.old < 1e-4)) break
-    # on the first iteration, expand argument values for affinity() or mosaic()
+    # On the first iteration, expand argument values for affinity() or mosaic()
     # (e.g. convert pH = c(0, 14) to pH = seq(0, 14, 256) so that it has the same length as the IS values)
-    # we don't do this if aout$vals is NA 20190731
+    # We don't do this if aout$vals is NA 20190731
     if(niter==1 & !all(is.na(aout$vals))) {
       if(thisfun=="affinity") for(i in 1:length(aout$vals)) {
         aout.save$args[[i]] <- aout$vals[[i]]
@@ -204,13 +204,13 @@ solubility_calc <- function(aout, dissociate = NULL, find.IS = FALSE, in.terms.o
         }
       }
     }
-    # recalculate the affinity using the new IS
+    # Recalculate the affinity using the new IS
     aout <- suppressMessages(do.call(thisfun, list(aout.save, IS = IS)))
     if(thisfun=="mosaic") aout <- aout$A.species
     niter <- niter + 1
   }
 
-  # do we want the solubility expressed in terms of
+  # Do we want the solubility expressed in terms of
   # something other than the first basis species? 20190123
   if(!is.null(in.terms.of)) {
     # write the reaction between the basis species and the new species
@@ -221,12 +221,12 @@ solubility_calc <- function(aout, dissociate = NULL, find.IS = FALSE, in.terms.o
     loga.balance <- loga.balance - log10(coeff)
   }
 
-  # make the output (we don't deal with normalized formulas yet, so for now m.balance==n.balance)
-  # indicate the function used to make this output 20190525
+  # Make the output (we don't deal with normalized formulas yet, so for now m.balance==n.balance)
+  # Indicate the function used to make this output 20190525
   aout$fun <- "solubility"
-  # add names to loga.equil 20190731
+  # Add names to loga.equil 20190731
   names(loga.equil) <- aout$species$name
-  c(aout, list(balance=bout$balance, m.balance=bout$n.balance, n.balance=bout$n.balance, in.terms.of=in.terms.of,
-    loga.balance=loga.balance, Astar=loga.equil, loga.equil=loga.equil))
+  c(aout, list(balance = bout$balance, m.balance = bout$n.balance, n.balance = bout$n.balance, in.terms.of = in.terms.of,
+    loga.balance = loga.balance, Astar = loga.equil, loga.equil = loga.equil))
 }
 
